global.config = require('./config.js');
express = require('express'),
session = require('express-session'),
path = require('path'),
logger = require('morgan'),
os = require('os'),
bodyParser = require('body-parser'),
cookieParser  = require('cookie-parser'),
mysql=require('mysql'),
engine=require('./engine.js'),
app = express();

/**
 * @alias session
 */
app.use(function(req,res,next){
    if(!req.headers.cookie && req.query.token){
        console.log('cookie hack',req.query.token);
        req.headers.cookie='connect.sid='+req.query.token;
    }
    res.header('Access-Control-Allow-Origin', '*');
    res.header('Access-Control-Allow-Credentials', true);
    res.header('Access-Control-Allow-Methods', 'POST, GET, PUT, DELETE, OPTIONS');
    res.header('Access-Control-Allow-Headers', 'Content-Type');
    next();
},session({
    cookie: {
        expires:new Date(Date.now() + 8*60*60*1000),
        maxAge:8*60*60*1000,
        httpOnly: false,
        path:"/"
    },
    secret:config.secret,
    saveUninitialized: true,
    unset:'destroy',
    resave:true ,
    store: new (require('express-mysql-session'))(
		(function(){
			var t=config.db;
			t.port = 3306,
			t.schema = {
				tableName: config.db.tables_prefix+'sessions',
				columnNames: {
					session_id: 'session_id',
					expires: 'expires',
					data: 'data'
				}
			}
			return t;
		})()
	)

}));

app.use(logger('dev'));
app.use(bodyParser.json({limit: '50mb'}));
app.use(bodyParser.urlencoded({ extended: false, limit: '50mb' }));
app.use(cookieParser(config.secret));
app.set('Charset','UTF-8');
app.use(require('less-middleware')(path.join(__dirname, 'templates/style'), {
      //debug:true,
      dest:path.join(__dirname, 'public'),
      preprocess: {
        path: function(pathname, req) {
          return pathname.replace(path.sep+'css'+path.sep, path.sep);
        }
      },
      force: true
}));
app.use(express.static(path.join(__dirname, 'public')));

// view engine setup
app.set('views', path.join(__dirname, 'templates'));
app.set('view engine', 'jade');
app.use('/', engine.router);



app.use(function(req, res, next) {
  var err = new Error('Not Found');
  err.status = 404;
  next(err);
});
app.use(function(err, req, res, next) {
    res.status(err.status || 500);
    console.error(err);
    res.render('error', {
        title:'Error',
        message: err.message,
        error: err
    });
});

var host;
var ifaces=os.networkInterfaces();
for(ifname in ifaces){
  var alias = 0;
  ifaces[ifname].forEach(function (iface){
    if ('IPv4' !== iface.family || iface.internal !== false)return;
    if(alias < 1) host=iface.address;
  });
};


var server = app.listen(config.port, function () {
    console.info('Apifonica example app listening at http://'+ host+':'+config.port);
});

if(!config.develop){
    const https = require('https');
    const fs = require('fs');
    const options = {
        key: fs.readFileSync('./keys/server.key'),
        cert: fs.readFileSync('./keys/server.crt')
    };
    server = https.createServer(options, app).listen(config.https_port,function(){
        console.info('Apifonica example app listening at https://'+ host+':'+(config.https_port));
    });
}


exports=this;