﻿**********************************************************************
Setting up Apifonica WebSDK packet (Version 2.2.3)
======================================================================
(c) Copyright 2017 MTT Oy. 
**********************************************************************

#### About

This WebSDK provides an easy way to use Apifonica service with REST APIs, to rent numbers, to create XML, and do other common server-side programming tasks. 
The WebSDK consists of two parts: the server-side path and the WebPhone demo project.  
The Server-side path of WebSDK uses for monitoring and connection between Mobile (iOS/Android), WebPhone devices and Apifonica service. 
The WebPhone demo project contains functions to rent phone number, make VoIP phone calls in the customer web browser.

#### Requirements

* npm
* Node.js (ver.5.x+)
* MySQL (ver.5.x+) or PostgreSQL

#### Main files of server-side packet 

User example for the Server-side contains: 
	`node_modules/apico_v2/` ([Readme](node_modules/apico_v2/readme.md))

1.Main files: 
	[app.js](app.js), [engine.js](engine.js), [db.js](db.js) for 'mysql' database or [pg.js](pg.js) for 'postgresql'. 

2.Router with api example: 
	[routes/index.js](routes/index.js)


#### Server-side Setup

1. Make sure you have the following packages installed: 
[node.js ](https://nodejs.org/en/download/) and [mysql](http://dev.mysql.com/downloads/mysql/)

2. Install **Forever** servicein console mode. Forever will make your server persistent, so that once you log off it will still run. It will restart even if node.js throws an error.
	```
	sudo npm install forever --global
	```

3. Copy and unpack the WebSDK archive package directly to the server where WebSDK needs to be deployed.

Note: If you are re-installing WebSDK over an existing installation, save the existing 'keys' folder and config.js file to a backup location. 
      When re-installing WebSDK, copy the 'keys' folder and config.js file from the backup to the server's root folder.

4. Using console mode, open the project’s root folder, enter the command **npm install** and wait until all the modules are installed:
	```
	cd apifonica_javascript_websdk
	npm install
	```	

5. Register on the apifonica.com web site, copy your 'AccountSID' and 'AuthToken' codes from the web site's API access section when you are logged in (as shown in the figure below) and enter these values into the config.js file: sid (AccountSID), token (AuthToken): 
	```
          sid: "accxx-xxxx-xxxx", 
        token: "autxx-xxx-xxxx" 
	```

6. Set your domain name and ports for http, https conections in the [config.js](config.js) file: 
	```
	// Deploy address
	domain:develop?"http://127.0.0.1:8000":"http://websdk.yourserver.com:8000",
	// port used to http
	port:8000,
	// port used to https
	https_port:8080,
	```

7. Open [package.json](package.json) file and correct your domain name:
	```
    "url": "http://websdk.yourserver.com:8000/",
    "sampleUrl": "http://websdk.yourserver.com:8000/",
	```

8. For first-time installations, create a self-signed certificate from the project root using console mode: 
	```
    cd apifonica_javascript_websdk
    mkdir keys
    cd keys
    openssl genrsa -des3 -out server-tmp.key
    openssl rsa -in server-tmp.key -out server.key
    openssl req -new -key server.key -out server.csr
    openssl x509 -req -days 3650 -in server.csr -signkey server.key -out server.crt
	```
	
9. Set up the MySQL database. 
   9.1. Using console mode, run these commands: 
	```
   	mysql –uUSERNAME –pPASSWORD;

	CREATE DATABASE IF NOT EXISTS websdk_example;

	USE websdk_example;

	CREATE TABLE IF NOT EXISTS `websdk_sessions` (
	    `session_id` VARCHAR(64) NOT NULL PRIMARY KEY,
	    `data` LONGTEXT NOT NULL,
	    `expires` integer
	);

	CREATE TABLE IF NOT EXISTS `websdk_number` (
	    `id` SERIAL PRIMARY KEY NOT NULL,
	    `number` varchar(16) NOT NULL UNIQUE,
	    `password` varchar(64) NOT NULL,
	    `number_sid` varchar(64) NOT NULL,
	    `user_id` integer REFERENCES number ON DELETE SET NULL,
	    `token` varchar(128),
	    `push_id` varchar(512)
	);

	CREATE TABLE IF NOT EXISTS `websdk_users` (
	    `id` SERIAL PRIMARY KEY NOT NULL ,
	    `login` varchar(64) NOT NULL UNIQUE,
	    `pass` varchar(64) NOT NULL,
	    `app_id` varchar(128)
	);
	```   

   9.2. Add one user with his/her account to the 'websdk_users' table: 
	```
	INSERT INTO websdk_users (login,pass,app_id) VALUES ( "login", "password", "Application_SID_code");
	```   

	where  "Application_SID_code" is your real 'Application SID' code value. The "Application SID" code identifier can be created in the Applications section of the Apifonica account web site (https://account.apifonica.com).
  

   9.3. Exit from mySQL service: 
	```
	exit;
 	```   

Note: Also you can create the database with the tables ready for using of SQL schema in [my_schema.sql](my_schema.sql) file:
   	```
        mysql –uUSERNAME –pPASSWORD < my_schema.sql;
	```   
	
   Remember: You must add at least one user with his/her account to the 'websdk_users' table. 

10. Edit the [config.js](config.js) file and insert your MYSQL settings:
	```
	db:{
		host	: 'localhost',
		user	: 'USERNAME',
		password: 'PASSWORD',
		database: 'websdk_example'
	},
	```
	
11. Edit the [config.js](config.js) file and set the active user ID (from the ‘websdk_users’ table) to identify the user whose rented numbers will be synchronized with the Apifonica server:
	```
	default_user:develop?1:2,
	```
	
12. Edit the [config.js](config.js) file and insert the 'sid' and 'token' codes taken from your Apifonica account (https://account.apifonica.com): 
	```
	apifo:{
		sid	: 'AccountSID',
		token	: 'AuthToken'
	},
	```



#### Starting the Server-side Service:

1. Start your server using commands in console mode:
	```
	cd apifonica_javascript_websdk
	forever start app.js
	``` 

Note: You can also use the **node app.js** command to start the server, but this command does not guarantee server restart after it goes offline: 
	```
	node app.js
	```  
	
2. Use this command to preview the log files generated by running forever processes:
	```
	forever logs
	``` 

3. Use **forever list** to verify that the server process is running. You need to make sure that no other process with the same name is running at the same time:
	```
	forever list
	``` 

4. To restart your server, use this command:
	```
	forever restart {process uid}
	```  
	where the 'process_uid' can be seen by running the **forever list** command. 
 

5. To stop the previously started server processes use this command in console mode:
	```
	forever stop app.js
	``` 

6. Open this HTML page in your browser to view a server-side management example:
	```
	https://websdk.yourserver.com:8080/register
	```	

7. Open this HTML page in your browser to see a WebPhone (WebRTC) demonstration:  
	```
	https://websdk.yourserver.com:8080/
	```	

	If you have gone through all the topics, you can look at the WebSDK login form for a WebPhone demonstration.

 
Note: Full list of available Forever commands for can be found at:  
	```
	https://www.npmjs.com/package/forever
	```

#### Server-side Management Example 

1. Here is an HTML page which demonstrates server management and its relation to MobileSDK applications and the apifonica.com server:  
	```
	https://websdk.yourserver.com:8080/register
	```

2. Depending your server's security settings, a window may pop up prompting you to enter a login and password in order to access the page that allows you to enter the authorization details issued to you by your server administrator.

3. The page displays the current values of your user identifiers (AccountSID / AuthToken), depending on the details shown in your Apifonica account on https://account.apifonica.com web site:  
	```
	Sid:  'AccountSID'  
	Token: 'AuthToken'  
	```

These values can then be compared with the user data stored in your Apifonica account. 
The page (https://websdk.yourserver.com:8080/register) also displays the current list of available accounts for iOS/Android applications with pre-installed MobileSDK packages to connect to. 

4. You can now enter the available commands for the WebSDK server side.

Note: The current demo case with the HTML page is provided for demonstration purposes only. Depending on your server security policies, it may be strongly unrecommended to use this example in the live mode!


#### List of Available Commands for Server-side Management  

1. Create a new account to connect WebPhone/iOS/Android applications with MobileSDK packages installed, to rent a phone number or to use already rented phone numbers to make voice calls. The user enters his/her desired username and password, sets the "Application SID" code for which a new account will be created to connect with iOS/Android applications and clicks the "Register" button. 

Note: The "Application SID" code identifier can be created in the Applications section of the Apifonica Account web site (https://account.apifonica.com). If you are planning to have multiple iOS/Android applications, it is possible to create an "Application SID" identifier for each of them.

2. Synchronize the rented numbers stored on the WebSDK server with the rented numbers stored in your account on apifonica.com. This command can be executed by pressing the "Sync" button. It is recommended to perform this command immediately after the first launch of the WebSDK server. As a result, the list of rented numbers will be displayed under the "Sync" button. Note that there is no indication when the synchronization is finished. 

Note: This command synchronizes the data in the MySQL database with the rented numbers only for the current user ID specified in the config.js file: 
	```
	default_user:develop?1:2
	```

#### WebPhone (WebRTC) Demonstration

1. To view the WebPhone (WebRTC) demonstration, open this URL in your web browser: 
	```
	https://websdk.yourserver.com:8080/
	```	

You should see the login form for the WebPhone demonstration.

2. Enter the login and password associated with the user ID used by default in Server-side Management and click the "login" button.
3. Click the "Food Basket" button to rent a new phone number.
4. Select any rented phone number and click the "Use" button or click the "Get available numbers" button to rent a new number.
5. You can now make any phone call.
	

#### Setting Up the WebPhone (WebRTC) Demonstration

1. Readme file: public/apifojs/readme.md
2. HTML file: templates/callout.jade
3. CSS file: templates/style/callout.less
4. JS file: public/js/callout.js
5. Interface elements in folder: public/img
6. Main files: app.js, engine.js, config.js
7. Router with API example: routes/index.js


