﻿var http=require('http'),
    https=require('https'),
    fs=require('fs'),
    request=require('request'),
    path=require('path');
/**
 * apifonica
 * @constructs apifonica
 * @param {string} account_sid - your Apifonica api login
 * @param {string} client_secret - your Apifonica api password
 */
module.exports=function(account_sid,client_token){
  if(!account_sid || !client_token){
    throw new Error('Invalid arguments list, client_id client_secret required!');
  }
  this.debug=true;
  this.predprod=false;
  this.use_app=null;
  //Private area
  {
      var token,app=this,
        // Set Apifonica url for REST API requests
        baseurl="https://api.apifonica.com";

    function api_send(input,callback) {
      var req = {}
      if(!input.url){
        err = new Error('URL required!');
        throw err;
      }else{
        req.url=baseurl+"/"+input.url;
      }
      req.json = input.data||{};
      req.method = input.type || input.method || "POST";
      req.headers=input.headers||{};
      req.headers.accept='*/*';
      req.headers['Content-Type']='application/json';
      req.auth = {
        user:account_sid,
        pass:client_token
      }
      console.log(req);
      request(req,function(err,response,body){
        console.log(response.statusCode, body);
        if(err || response.statusCode >=400){
          console.log(input);
          console.error(err||body);
          callback(err||body);
        }else{
          callback(null,body)
        }
      });
    }
  }

  //Public area
  {
    this._drop_token=function(){ token='invalid'; }


    //MESSAGING
    /**
     * @param {object} input
     * @param {string} input.from - A phone number to be used as a message sender. This number must be acquired from Apifonica.
     * @param {string} input.to - Recipient’s phone number.
     * @param {string} input.text - Text of the message. Max 160 Latin characters or 70 Unicode characters.
     * @param {function} cb - callback
     */
    this.create_message=function(input,cb){
      api_send({
        url:"v2/accounts/"+account_sid+'/messages',
        type:"POST",
        data:input
      },cb);
    }
    /**
     * @param {object} input
     * @param {string} [input.from] - A phone number to be used as a message sender. This number must be acquired from Apifonica.
     * @param {string} [input.to] - Recipient’s phone number.
     * @param {string} [input.direction] - Either inbound or outbound.
     * @param {string} [input.status] - Can be one of the following: queued, sent, received, delivered, failed.
     * @param {string} [input.message_sid] - Message identifier.
     * @param {string} [input.account_sid] - Account identifier.
     * @param {string} [input.created] - Message creation date.
     * @param {number} [offset=0] - A parameter telling where to start returning records from the entire set of results. If you don't include this parameter, the default is to start at record number 0 and then return the number of records specified with the limit parameter.
     * @param {number} [limit=20] - A parameter describing how many records you want to return all at once. The default and maximum value is 20.
     * @param {function} cb - callback
     */
    this.get_messages_list = function(input,cb){
      api_send({
        url:"v2/accounts/"+account_sid+'/messages',
        type:"GET",
        data:input
      },cb);
    }


   /**
     * @param {object} input
     * @param {string} input.message_sid - =
     * @param {function} cb - callback
     */
    this.get_message = function(input,cb){
      api_send({
        url:"v2/accounts/"+account_sid+'/messages/'+input.message_sid,
        type:"GET"
      },cb);
    }








    //Voicing
    //============Voicing============================
    this.modify_live_call=function(input,callback){
      if(!input.number)return callback('Invalid number '+input.number);
      api_send({
        url:"v2/accounts/"+account_sid+"/calls/"+input.call_sid,
        data:{
          controller:input.controller
        }
      },function(err,res){
        if(err)return callback(err);
        callback(null,res);
      });
    }

    this.get_call=function(input,callback){
      if(!input.call_sid)return callback('Invalid call_sid '+input.call_sid);
      api_send({
        url:"v2/accounts/"+account_sid+"/calls/"+input.call_sid,
        method:"GET"
      },function(err,res){
        if(err)return callback(err);
        callback(null,res);
      });
    }



    //Applications
	/**
     * @param {object} input
     * @param {object} [input.name] - 
     * @param {object} [input.controller] - 
     * @param {function} cb - callback
     */
    this.create_application = function(input,callback){
      api_send({
        url:"v2/accounts/"+account_sid+"/applications",
        method:"POST",
		data:input
      },function(err,res){
        if(err)return callback(err);
        callback(null,res);
      });
    }
	
	
	
	//Numbering
    /**
     * @param {object} input
     * @param {function} cb - callback
     */
    this.get_available_countries = function(input,callback){
      api_send({
        url:"v2/accounts/"+account_sid+"/availableNumbers",
        method:"GET"
      },function(err,res){
        if(err)return callback(err);
        callback(null,res);
      });
    }

    /**
     * @param {object} input
     * @param {string} input.country_code - Country code according to ISO 3166-1 alpha-2 or “international”
     * @param {string} [input.number] -	Phone number in E.164 format.
     * @param {string} [input.type] - Number type. Can be “local”, “mobile”, “toll-free” or “sip”.
     * @param {string} [input.voice] - Availability of voice communications for this number. Can me “true” or “false”.
     * @param {string} [input.messaging] - Availability of messaging for this number. Can me “true” or “false”.
     * @param {function} cb - callback
     */
    this.get_available_numbers = function(input,callback){
      if(input.location)input.country_code = input.location;  
      if(!input.country_code)return callback('Invalid country_code '+input.country_code);
      var query = '?';
      for(var i in input){
        if(i == 'country_code')query+= 'countryCode='+input[i]+'&';
        else query+= i+'='+input[i]+'&';
      }

      api_send({
      //url:"v2/accounts/"+account_sid+"/availableNumbers/"+input.country_code+query,
        url:"v2/accounts/"+account_sid+"/availableNumbers/"+input.country_code+query+"&voice=true",
        method:"GET"
      },function(err,res){
        if(err)return callback(err);
        callback(null,res);
      });
    }

    /**
     * @param {object} input
     * @param {string} input.number -	Phone number in E.164 format.
     * @param {string} [input.call_app_sid] - Link to exist Application. Will be use for Call manipulations. Will get controller, method for calls.
     * @param {string} [input.msg_app_sid] - Link to exist Application. Will be use for Message manipulations. Will get controller, method for Messages events.
     * @param {function} cb - callback
     */
    this.rent_number = function(input,callback){
      if(!input.number)return callback('Invalid number '+input.number);
      api_send({
        url:"v2/accounts/"+account_sid+"/numbers",
        method:"POST",
        data:input
      },function(err,res){
        if(err)return callback(err);
        callback(null,res);
      });
    }

    /**
     * @param {object} input
     * @param {string} [input.page] - Page number starting with 0.
     * @param {string} [input.limit] - Number of returned entries per page (default and maximum 20).
     * @param {string} [input.number] - Phone number in E.164 format.
     * @param {string} [input.country_code] - Country code according to ISO 3166-1 alpha-2 or “international”.
     * @param {string} [input.country_name] - Full name of the country.
     * @param {string} [input.type] - Number type. Can be “local”, “mobile”, “toll-free” or “sip”.
     * @param {string} [input.voice] - Availability of voice communications for this number. Can me “true” or “false”.
     * @param {string} [input.messaging] - Availability of messaging for this number. Can me “true” or “false”.
     * @param {function} cb - callback
     */
    this.get_numbers_list = function(input,callback){
      var query = '?';
      for(var i in input)query+= i+'='+input[i]+'&';
      api_send({
        url:"v2/accounts/"+account_sid+"/numbers"+query,
        method:"GET",
        data:input
      },function(err,res){
        if(err)return callback(err);
        callback(null,res);
      });
    }

    /**
     * @param {object} input
     * @param {string} input.number_sid - URL which will be used to make a request for an XML controller file for incoming calls or messages.
     * @param {string} [input.app_sid] - URL which will be used to make a request for an XML controller file for incoming calls or messages.
     * @param {string} [input.mode] - URL which regulates the phone number mode ("end_point", "sip_phone", "xml")
     * @param {string} [input.sip_password] - SIP password which will be used to make SIP calls. Works only for international numbers with type = sip.
     * @param {function} cb - callback
     */
    this.update_number = function(input,callback){

      var input2 = {};
      for(var i in input){
        if(i != 'number_sid') 
	   input2[i] = input[i];
      }
      api_send({
        url:"v2/accounts/"+account_sid+"/numbers/"+input.number_sid,
        method:"PUT",
        data:input2
      },function(err,res){
        if(err)return callback(err);
        callback(null,res);
      });
    }

    /**
     * @param {object} input
     * @param {string} input.number_sid - URL which will be used to make a request for an XML controller file for incoming calls or messages.
     * @param {string} input.account_sid - Target account identifier. Number will be moved to this account.
     * @param {function} cb - callback
     */
    this.move_number = function(input,callback){

      api_send({
        url:"v2/accounts/"+account_sid+"/numbers/"+input.number_sid,
        method:"PUT",
        data:input
      },function(err,res){
        if(err)return callback(err);
        callback(null,res);
      });
    }


    /**
     * @param {object} input
     * @param {string} input.number_sid - URL which will be used to make a request for an XML controller file for incoming calls or messages.
     * @param {function} cb - callback
     */
    this.get_number = function(input,callback){

      api_send({
        url:"v2/accounts/"+account_sid+"/numbers/"+input.number_sid,
        method:"GET"
      },function(err,res){
        if(err)return callback(err);
        callback(null,res);
      });
    }

    /**
     * @param {object} input
     * @param {string} input.number_sid - URL which will be used to make a request for an XML controller file for incoming calls or messages.
     * @param {function} cb - callback
     */
    this.delete_number = function(input,callback){
      if(!input.number_sid)return callback('number_sid required!');
      api_send({
        url:"v2/accounts/"+account_sid+"/numbers/"+input.number_sid,
        method:"DELETE"
      },function(err,res){
        if(err)return callback(err);
        callback(null,res);
      });
    }

    /**
     * @param {object} input
     * @param {string} [input.number] -	Phone number in E.164 format.
     * @param {string} [input.name] - Friendly name.
     * @param {function} cb - callback
     */
    this.get_available_sip_numbers = function(input,callback){
      var query = '?';
      for(var i in input)query+= i+'='+input[i]+'&';
      api_send({
        url:"v2/accounts/"+account_sid+"/availableNumbers/virtual",
        method:"GET"
      },function(err,res){
        if(err)return callback(err);
        callback(null,res);
      });
    }

    /**
     * @param {object} input
     * @param {string} input.number_sid - URL which will be used to make a request for an XML controller file for incoming calls or messages.
     * @param {function} cb - callback
     */
    this.get_sip_number = function(input,callback){

      api_send({
        url:"v2/accounts/"+account_sid+"/availableNumbers/virtual/"+input.number_sid,
        method:"GET"
      },function(err,res){
        if(err)return callback(err);
        callback(null,res);
      });
    }







  }
}